#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['fastgenomics', 'fastgenomics.external']

package_data = \
{'': ['*'], 'fastgenomics': ['schemes/*', 'templates/*']}

install_requires = \
['deprecated~=1.2.4',
 'jsonschema~=2.6',
 'Jinja2~=2.9',
 'get_version~=2.0.1',
 'setuptools']

extras_require = \
{'doc': ['sphinx', 'sphinx_rtd_theme', 'sphinx-autodoc-typehints'],
 'test': ['pytest >=4', 'anndata >=0.6']}

setup(name='fastgenomics',
      version='2.0.1',
      description='FASTGenomics python helper',
      author='FASTGenomics Team',
      author_email='contact@fastgenomics.org',
      url='https://github.com/fastgenomics/fastgenomics-py',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='~=3.6',
     )
